from typing import Union

import pandas as pd

from format_call import OUTPUT_FUNCTION_MAPPING
from constants import BASE_URL


class Api():
    '''
    Define the structure of APIs functionnality
    All APIs should follow this structure
    '''
    def __init__(
        self, 
        api_type: str,
        export_format: str = 'JSON', 
        root_url: str = BASE_URL,
        parameters: Union[str, dict] = None, 
        _id: str = None,
        version_format: float = 2.1,
        language: str = 'en',
        path: str = '.',
    ) -> None:
        
        self.root_url = root_url
        self.export_format = export_format
        self.parameters = parameters
        
        self._id = _id
        self.version = version_format
        self.language = language
        self.path = path
        self.api_url = get_url(api_type, self)
    
    def api_call(self) -> Union[dict, pd.DataFrame]:
        api_function = OUTPUT_FUNCTION_MAPPING[self.export_format]
        return api_function(
            f'{self.root_url}/api/{self.api_url}', 
            self.parameters
        )

    
def get_url(_type, self):
    
    url_mapping = {
        'codelist':
            f'CodeLists/{self._id}/exports/{self.export_format}/{self.version}',
        'content_configuration': 
            'ContentConfigurations',
        'content_configuration_identifier': 
            f'ContentConfigurations/{self._id}',
        'dcat_dataset_description': 
            f'Datasets/{self._id}/{self.language}/description',
        'dcat_dataset_information': 
            f'Datasets/{self._id}/{self.language}/distributions',
        'data_structure': 
            f'DataStructures/{self._id}/{self.language}',
        'nomenclature_path_nodes': 
            f'Nomenclatures/Childnodes/{self._id}/{self.language}/{self.path}',
        'nomenclature_one_level': 
            f'Nomenclatures/{self._id}/levelexport/CSV',
        'nomenclature_multiple_levels': 
            f'Nomenclatures/{self._id}/multiplelevels/CSV',
        'nomenclature_search': 
            f'Nomenclatures/{self._id}/search',
        'agents_list': 
            'Agent',
        'agent_id': 
            f'Agent/{self._id}',
        'dataset_list': 
            'Dataset',
        'dataset_id_distributions': 
            f'Dataset/{self._id}/distributions',
        'dataset_id': 
            f'Dataset/{self._id}',
        'dataset_identifier': 
            f'Datataset/identifier/{self._id}',
        'dataset_identifier_distributions': 
            f'Datataset/identifier/{self._id}/distributions',
        'distributions_list': 
            f'Distribution',
        'distribution_id':  
            f'Distribution/{self._id}'
    }
    
    return url_mapping[_type]
