import requests

import pandas as pd
import pandasdmx as sdmx


def sdmx_api(api_url: str, params: str) -> pd.Series:
    '''
    API query for SDMX output
    Parameters:
        - api_url: url to query 
        - params: additional query parameters   
        
    Returns:
        - pd.Serie response
    '''
    req = sdmx.Request()
    print(f'{api_url}?{params}')
    response = req.get(url=f'{api_url}?{params}')
    return sdmx.to_pandas(response)[0][0]


def json_api(api_url: str, params: str) -> dict:
    '''
    API query for JSON output
    Parameters:
        - api_url: url to query 
        - params: additional query parameters
    
    Returns:
        - dictionnary response
    '''
    return requests.get(url = api_url, params = params).json()


def xlsx_api(api_url: str, params: str) -> pd.DataFrame:
    '''
    API query for XLSX output
    Parameters:
        - api_url: url to query 
        - params: additional query parameters
    
    Returns:
        - pd.DataFrame response
    '''
    # As dtype string to keep leading 0s and avoid decimals
    return pd.read_excel(f'{api_url}?{params}', dtype=str)


def csv_api(api_url: str, params: str) -> pd.DataFrame:
    '''
    API query for CSV output
    Parameters:
        - api_url: url to query 
        - params: additional query parameters
    
    Returns:
        - pd.DataFrame response
    '''
    # As dtype string to keep leading 0s and avoid decimals
    return pd.read_csv(f'{api_url}?{params}', dtype=str)


# Map the appropriate function based on the output type
OUTPUT_FUNCTION_MAPPING = {
    'JSON': json_api,
    'SDMX-JSON': json_api,
    'SDMX-ML': sdmx_api,
    'XLSX': xlsx_api,
    'CSV': csv_api
}
